/**
 * Configuration options for the SSE client
 */
export interface SSEClientOptions {
    /**
     * Headers to include in the SSE request
     */
    headers?: Record<string, string>;
    /**
     * Whether to include credentials in the request
     * @default true
     */
    withCredentials?: boolean;
    /**
     * Timeout for heartbeat in milliseconds
     * @default 21600000 (6 hours)
     */
    heartbeatTimeout?: number;
    /**
     * Maximum number of retry attempts
     * @default 10
     */
    maxRetryAttempts?: number;
    /**
     * Initial delay for retry in milliseconds
     * @default 1000
     */
    initialRetryDelay?: number;
    /**
     * Maximum delay for retry in milliseconds
     * @default 30000
     */
    maxRetryDelay?: number;
    /**
     * Whether to automatically reconnect on error
     * @default true
     */
    autoReconnect?: boolean;
    /**
     * Debug mode
     * @default false
     */
    debug?: boolean;
    /**
     * Whether to bypass SSL certificate validation
     * Only works in Node.js environments
     * @default false
     */
    skipSSLValidation?: boolean;
}
/**
 * A custom SSE client that supports headers and bypasses certificate issues
 */
export declare class SSEClient {
    private eventSource;
    private url;
    private options;
    private retryCount;
    private closeRequested;
    private eventListeners;
    private customEventTarget;
    /**
     * Creates a new SSE client
     * @param url The URL to connect to
     * @param options Configuration options
     */
    constructor(url: string, options?: SSEClientOptions);
    /**
     * Connects to the SSE endpoint
     * @returns The SSE client instance for chaining
     */
    connect(): SSEClient;
    /**
     * Closes the SSE connection
     */
    close(): void;
    /**
     * Adds an event listener for SSE events
     * @param eventName The name of the event to listen for
     * @param listener The event listener function
     * @returns The SSE client instance for chaining
     */
    on(eventName: string, listener: (event: any) => void): SSEClient;
    /**
     * Removes an event listener
     * @param eventName The name of the event
     * @param listener The event listener function to remove
     * @returns The SSE client instance for chaining
     */
    off(eventName: string, listener: (event: any) => void): SSEClient;
    /**
     * Removes all event listeners for a specific event
     * @param eventName The name of the event
     * @returns The SSE client instance for chaining
     */
    removeAllListeners(eventName?: string): SSEClient;
    /**
     * Creates a new SSE connection
     */
    private createConnection;
    /**
     * Adds an event listener to the event source
     * @param eventName The name of the event
     * @param listener The event listener function
     */
    private addEventSourceListener;
    /**
     * Handles the open event
     */
    private handleOpen;
    /**
     * Handles the error event
     * @param error The error that occurred
     */
    private handleError;
    /**
     * Reconnects to the SSE endpoint with exponential backoff
     */
    private reconnect;
}
/**
 * Singleton pattern for SSE connections
 */
export declare class SSEConnectionManager {
    private static instance;
    private connections;
    private constructor();
    /**
     * Gets the singleton instance
     * @returns The SSE connection manager instance
     */
    static getInstance(): SSEConnectionManager;
    /**
     * Gets or creates a connection for the given ID
     * @param url The URL to connect to
     * @param id A unique identifier for the connection
     * @param options Configuration options
     * @returns The SSE client instance
     */
    getConnection(url: string, id: string, options?: SSEClientOptions): SSEClient;
    /**
     * Closes a connection for the given ID
     * @param id The connection ID
     */
    closeConnection(id: string): void;
    /**
     * Closes all connections
     */
    closeAllConnections(): void;
}
/**
 * Gets or creates a connection for the given ID
 * @param url The URL to connect to
 * @param id A unique identifier for the connection
 * @param options Configuration options
 * @returns The SSE client instance
 */
export declare function getSSEConnection(url: string, id: string, options?: SSEClientOptions): SSEClient;
/**
 * Closes a connection for the given ID
 * @param id The connection ID
 */
export declare function closeSSEConnection(id: string): void;
/**
 * Closes all connections
 */
export declare function closeAllSSEConnections(): void;
/**
 * Sets up event listeners to clean up SSE connections when the page is unloaded
 */
export declare function setupSSECleanup(): void;
