"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SSEConnectionManager = exports.SSEClient = void 0;
exports.getSSEConnection = getSSEConnection;
exports.closeSSEConnection = closeSSEConnection;
exports.closeAllSSEConnections = closeAllSSEConnections;
exports.setupSSECleanup = setupSSECleanup;
const event_source_polyfill_1 = require("event-source-polyfill");
const debug_1 = require("./utils/debug");
/**
 * Default options for the SSE client
 */
const DEFAULT_OPTIONS = {
    withCredentials: true,
    heartbeatTimeout: 21600000, // 6 hours
    maxRetryAttempts: 10,
    initialRetryDelay: 1000,
    maxRetryDelay: 30000,
    autoReconnect: true,
    debug: false,
    skipSSLValidation: false
};
/**
 * Helper: delay for ms
 */
function delay(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
/**
 * A custom SSE client that supports headers and bypasses certificate issues
 */
class SSEClient {
    /**
     * Creates a new SSE client
     * @param url The URL to connect to
     * @param options Configuration options
     */
    constructor(url, options = {}) {
        this.eventSource = null;
        this.retryCount = 0;
        this.closeRequested = false;
        this.eventListeners = new Map();
        this.url = url;
        this.options = {
            ...DEFAULT_OPTIONS,
            headers: options.headers || {},
            ...options
        };
        // Set debug mode
        if (this.options.debug) {
            globalThis.__SSE_DEBUG_ENABLED = true;
        }
        // Create a custom event target for dispatching events
        this.customEventTarget = typeof window !== 'undefined'
            ? document.createElement('div')
            : new EventTarget();
        (0, debug_1.debugLog)('SSEClient: Created new client for URL:', url);
    }
    /**
     * Connects to the SSE endpoint
     * @returns The SSE client instance for chaining
     */
    connect() {
        if (this.eventSource) {
            (0, debug_1.debugWarn)('SSEClient: Already connected, closing existing connection');
            this.close();
        }
        this.closeRequested = false;
        this.createConnection();
        return this;
    }
    /**
     * Closes the SSE connection
     */
    close() {
        this.closeRequested = true;
        if (this.eventSource) {
            (0, debug_1.debugLog)('SSEClient: Closing connection');
            this.eventSource.close();
            this.eventSource = null;
        }
    }
    /**
     * Adds an event listener for SSE events
     * @param eventName The name of the event to listen for
     * @param listener The event listener function
     * @returns The SSE client instance for chaining
     */
    on(eventName, listener) {
        if (!this.eventListeners.has(eventName)) {
            this.eventListeners.set(eventName, new Set());
        }
        this.eventListeners.get(eventName).add(listener);
        // If already connected, add the listener to the event source
        if (this.eventSource) {
            this.addEventSourceListener(eventName, listener);
        }
        return this;
    }
    /**
     * Removes an event listener
     * @param eventName The name of the event
     * @param listener The event listener function to remove
     * @returns The SSE client instance for chaining
     */
    off(eventName, listener) {
        const listeners = this.eventListeners.get(eventName);
        if (listeners) {
            listeners.delete(listener);
            if (listeners.size === 0) {
                this.eventListeners.delete(eventName);
            }
        }
        return this;
    }
    /**
     * Removes all event listeners for a specific event
     * @param eventName The name of the event
     * @returns The SSE client instance for chaining
     */
    removeAllListeners(eventName) {
        if (eventName) {
            this.eventListeners.delete(eventName);
        }
        else {
            this.eventListeners.clear();
        }
        return this;
    }
    /**
     * Creates a new SSE connection
     */
    createConnection() {
        (0, debug_1.debugLog)('SSEClient: Creating new connection to:', this.url);
        try {
            // Create connection options
            const connectionOptions = {
                headers: this.options.headers,
                withCredentials: this.options.withCredentials,
                heartbeatTimeout: this.options.heartbeatTimeout
            };
            // Add SSL bypass if requested and in Node.js environment
            if (this.options.skipSSLValidation && typeof window === 'undefined') {
                try {
                    // Only import https in Node.js environment
                    const https = require('https');
                    connectionOptions.httpsAgent = new https.Agent({ rejectUnauthorized: false });
                    (0, debug_1.debugLog)('SSEClient: SSL certificate validation bypassed');
                }
                catch (e) {
                    (0, debug_1.debugWarn)('SSEClient: Failed to create https agent for SSL bypass:', e);
                }
            }
            this.eventSource = new event_source_polyfill_1.EventSourcePolyfill(this.url, connectionOptions);
            // Set up event handlers
            this.eventSource.onopen = this.handleOpen.bind(this);
            this.eventSource.onerror = this.handleError.bind(this);
            // Add event listeners for all registered events
            for (const [eventName, listeners] of this.eventListeners.entries()) {
                for (const listener of listeners) {
                    this.addEventSourceListener(eventName, listener);
                }
            }
        }
        catch (error) {
            (0, debug_1.debugError)('SSEClient: Error creating connection:', error);
            this.handleError(error);
        }
    }
    /**
     * Adds an event listener to the event source
     * @param eventName The name of the event
     * @param listener The event listener function
     */
    addEventSourceListener(eventName, listener) {
        if (!this.eventSource)
            return;
        this.eventSource.addEventListener(eventName, (event) => {
            try {
                (0, debug_1.debugLog)(`SSEClient: Received ${eventName} event:`, event);
                // Parse the event data if it's a string
                if (typeof event.data === 'string') {
                    try {
                        const parsedData = JSON.parse(event.data);
                        event.parsedData = parsedData;
                    }
                    catch (e) {
                        // If parsing fails, just use the raw data
                        (0, debug_1.debugWarn)(`SSEClient: Failed to parse event data as JSON:`, e);
                    }
                }
                // Call the listener with the event
                listener(event);
            }
            catch (error) {
                (0, debug_1.debugError)(`SSEClient: Error handling ${eventName} event:`, error);
            }
        });
    }
    /**
     * Handles the open event
     */
    handleOpen() {
        (0, debug_1.debugLog)('SSEClient: Connection opened');
        this.retryCount = 0;
        // Dispatch the open event
        const openEvent = new Event('open');
        this.customEventTarget.dispatchEvent(openEvent);
    }
    /**
     * Handles the error event
     * @param error The error that occurred
     */
    handleError(error) {
        (0, debug_1.debugError)('SSEClient: Connection error:', error);
        // Dispatch the error event
        const errorEvent = new CustomEvent('error', { detail: error });
        this.customEventTarget.dispatchEvent(errorEvent);
        // If close was requested, don't reconnect
        if (this.closeRequested) {
            (0, debug_1.debugLog)('SSEClient: Not reconnecting (close requested)');
            return;
        }
        // If auto reconnect is disabled, don't reconnect
        if (!this.options.autoReconnect) {
            (0, debug_1.debugLog)('SSEClient: Not reconnecting (auto reconnect disabled)');
            return;
        }
        // If max retry attempts reached, don't reconnect
        if (this.retryCount >= this.options.maxRetryAttempts) {
            (0, debug_1.debugError)(`SSEClient: Max retry attempts (${this.options.maxRetryAttempts}) reached, not reconnecting`);
            return;
        }
        // Reconnect with exponential backoff
        this.reconnect();
    }
    /**
     * Reconnects to the SSE endpoint with exponential backoff
     */
    async reconnect() {
        this.retryCount++;
        // Calculate delay with exponential backoff
        const delayMs = Math.min(this.options.initialRetryDelay * Math.pow(2, this.retryCount - 1), this.options.maxRetryDelay);
        (0, debug_1.debugLog)(`SSEClient: Reconnecting in ${delayMs}ms (attempt ${this.retryCount}/${this.options.maxRetryAttempts})`);
        // Wait for the delay
        await delay(delayMs);
        // If close was requested during the delay, don't reconnect
        if (this.closeRequested) {
            (0, debug_1.debugLog)('SSEClient: Not reconnecting (close requested during delay)');
            return;
        }
        // Create a new connection
        this.createConnection();
    }
}
exports.SSEClient = SSEClient;
/**
 * Singleton pattern for SSE connections
 */
class SSEConnectionManager {
    constructor() {
        this.connections = new Map();
    }
    /**
     * Gets the singleton instance
     * @returns The SSE connection manager instance
     */
    static getInstance() {
        if (!SSEConnectionManager.instance) {
            SSEConnectionManager.instance = new SSEConnectionManager();
        }
        return SSEConnectionManager.instance;
    }
    /**
     * Gets or creates a connection for the given ID
     * @param url The URL to connect to
     * @param id A unique identifier for the connection
     * @param options Configuration options
     * @returns The SSE client instance
     */
    getConnection(url, id, options = {}) {
        if (this.connections.has(id)) {
            (0, debug_1.debugLog)(`SSEConnectionManager: Reusing existing connection for ID ${id}`);
            return this.connections.get(id);
        }
        (0, debug_1.debugLog)(`SSEConnectionManager: Creating new connection for ID ${id}`);
        const client = new SSEClient(url, options).connect();
        this.connections.set(id, client);
        return client;
    }
    /**
     * Closes a connection for the given ID
     * @param id The connection ID
     */
    closeConnection(id) {
        if (this.connections.has(id)) {
            (0, debug_1.debugLog)(`SSEConnectionManager: Closing connection for ID ${id}`);
            this.connections.get(id).close();
            this.connections.delete(id);
        }
    }
    /**
     * Closes all connections
     */
    closeAllConnections() {
        (0, debug_1.debugLog)(`SSEConnectionManager: Closing all connections`);
        for (const [id, client] of this.connections.entries()) {
            client.close();
            this.connections.delete(id);
        }
    }
}
exports.SSEConnectionManager = SSEConnectionManager;
/**
 * Gets or creates a connection for the given ID
 * @param url The URL to connect to
 * @param id A unique identifier for the connection
 * @param options Configuration options
 * @returns The SSE client instance
 */
function getSSEConnection(url, id, options = {}) {
    return SSEConnectionManager.getInstance().getConnection(url, id, options);
}
/**
 * Closes a connection for the given ID
 * @param id The connection ID
 */
function closeSSEConnection(id) {
    SSEConnectionManager.getInstance().closeConnection(id);
}
/**
 * Closes all connections
 */
function closeAllSSEConnections() {
    SSEConnectionManager.getInstance().closeAllConnections();
}
/**
 * Sets up event listeners to clean up SSE connections when the page is unloaded
 */
function setupSSECleanup() {
    if (typeof window === 'undefined')
        return;
    // Clean up all SSE connections when the page is unloaded
    window.addEventListener('beforeunload', () => {
        (0, debug_1.debugLog)('SSECleanup: Cleaning up all SSE connections before page unload');
        closeAllSSEConnections();
    });
}
